/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.buttons.archive');

org.ametys.cms.buttons.archive.ArchiveMenu = function(id, configuration, pluginName) 
{
	org.ametys.cms.buttons.archive.ArchiveMenu.superclass.constructor.call(this, id, configuration, pluginName);
	
	this._currentContentsId = [];
	this._allRightContentsId = []
};

Ext.extend(org.ametys.cms.buttons.archive.ArchiveMenu, org.ametys.cms.ribbon.button.Menu, {});

/**
 * @private
 * @property {String[]} _currentContentsId The contents id currently selected or null. 
 */
org.ametys.cms.buttons.archive.ArchiveMenu.prototype._currentContentsId;
/**
 * @private
 * @property {String[]} _allRightContentsId The contents id currently selected that are allowed to be archived.
 */
org.ametys.cms.buttons.archive.ArchiveMenu.prototype._allRightContentsId;

org.ametys.cms.buttons.archive.ArchiveMenu.prototype.createControl = function(size, colspan)
{
    var element = org.ametys.cms.buttons.archive.ArchiveMenu.superclass.createControl.call(this, size, colspan);
    
    var menuCount = parseInt(this.getConfiguration()["menu-size"]); 
    if (menuCount > 0)
    {
        for (var i=0;  i < menuCount; i++)
        {
            element._menuItems["menu-" + i].setHandler(
                function (button) 
                {
                    eval(this.ribbonControl.getConfiguration()["menu-" + this.index + "-action"] + "(button, this.ribbonControl.getConfiguration(), this.ribbonControl.getPluginName(), this.ribbonControl._allRightContentsId, this.index);");
                }
            ); 
        }
    }
    
    return element;
}

org.ametys.cms.buttons.archive.ArchiveMenu.act = function (controlId, configuration, pluginName)
{
    var control = org.ametys.ribbon.RibbonManager.getInstance().getControls()[controlId];
    var index;
    if (!control.getElements().get(0).pressed)
    {
        index = configuration["non-toggled-item-index"];
    }
    else
    {
        index = configuration["toggled-item-index"];
    }
    
    eval(configuration["menu-" + index + "-action"] + "(control.getId(), control.getConfiguration(), control.getPluginName(), control._allRightContentsId, index);");
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype._areTheSame = function (tab1, tab2)
{
    if (tab1 == tab2)
    {
        return true;
    }

    if (tab1 == null || tab2 == null || tab1.length != tab2.length)
    {
        return false;
    }   
    
    for (var i=0; i < tab1.length; i++)
    {
        if (tab2.indexOf(tab1[i]) == -1)
        {
            return false;
        }
    }
    
    return true;
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype._enterRefreshingMode = function ()
{
    this.getElements().each(
        function(element) 
        { 
            element.setDisabled(true);
            element.toggle(false);
            element.refreshing();
        }
    );
    this.updateTooltip(this.getConfiguration()["refresh-description"]);
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype.onMessage = function (messages)
{
    var alreadyRefreshed = false;
    
    // SELECTION
    for (var i=messages.length - 1; i >= 0; i--)
    {
        if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
        {
            var targets = messages[i].getTargets();
            var allTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
            var allTargetsId = this._convertToId(allTargets);
            
            // No content selected
            if (allTargetsId.length == 0)
            {
                this._currentContentsId = [];
                
                var iconSmall = context.contextPath + this.getConfiguration()['icon-small'];
                var iconLarge = context.contextPath + this.getConfiguration()['icon-medium'];
                
                this.getElements().each(
                        function (element) 
                        {
                            element.iconSmall = iconSmall;
                            element.iconLarge = iconLarge;
                        }
                    );
                
                this.getElements().each(function(element) { element.updateIcon(); });
                this.updateTooltip(this.getConfiguration()["no-selection-description"], iconLarge, null);
                
                this.getElements().each(function(element) { element.setDisabled(true); element.toggle(false); });
            }
            // At least one content selected
            else
            {
                if (!this._areTheSame(this._currentContentsId, allTargetsId))
                {
                    this._currentContentsId = allTargetsId;
                    this._enterRefreshingMode();
                    org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: allTargetsId});
                }
            }

            break;
        }
    }
    
    // WORKFLOW, LOCK, METADATA
    for (var i=0; i < messages.length; i++)
    {
        if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_WORKFLOWCHANGED
            || messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_WORKFLOWCHANGING
            || messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_LOCKCHANGED
            || messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_METADATACHANGED
            || messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_MINORMETADATACHANGED)
        {
            var targets = messages[i].getTargets();
            var allTargets = this._convertToId(org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget()));
            
            var currentSelection = org.ametys.ribbon.RibbonManager.getInstance().getCurrentSelectionTargets();
            var currentSelectionIds = this._convertToId(org.ametys.messagebus.message.MessageTargetHelper.findAll(currentSelection, this.checkTarget()));
            
            // No content selected
            if (currentSelectionIds.length == 0)
            {
                continue;
            }
            // At least one content selected
            {
                var stop = false;
                for (var t=0; !stop && t < allTargets.length; t++)
                {
                    for (var c=0; !stop && c < currentSelectionIds.length; c++)
                    {
                        if (currentSelectionIds[c] == allTargets[t])
                        {
                            this._enterRefreshingMode();
                            
                            if (messages[i].getType().getName() != org.ametys.ribbon.RibbonManager.EVENTTYPE_WORKFLOWCHANGING)
                            {
                                if (alreadyRefreshed == false)
                                {
                                    alreadyRefreshed = true;
                                    org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: currentSelectionIds});
                                }
                            }

                            stop = true;
                        }
                    }
                }
            }
        }
    }
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype._convertToId = function (targets)
{
    var targetsIds = [];
    for (var i = 0; i < targets.length; i++)
    {
        targetsIds.push(targets[i].getParameters()['id']);
    }
    return targetsIds;
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype.checkTarget = function ()
{
    return function (target) { return target.getType() == 'content'; };
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype._allRightTooltip = function(tooltip, paramPrefix, params, isScheduled)
{
    var index;
    if (!isScheduled)
    {
        index = this.getConfiguration()["non-toggled-item-index"];
    }
    else
    {
        index = this.getConfiguration()["toggled-item-index"];
    }
    
    var count = parseInt(params[paramPrefix + "-content-size"]);
    if (count > 0)
    {
        if (tooltip != "")
        {
            tooltip += "<br/><br/>";
        }
        
        tooltip += this.getConfiguration()["menu-" + index + "-" + paramPrefix + "-start-description"];
        for (var i=0; i < count; i++)
        {
            if (i != 0) 
            {
                tooltip += ", ";
            }
            tooltip += params[paramPrefix + "-content-" + i];
        }
        tooltip += this.getConfiguration()["menu-" + index + "-" + paramPrefix + "-end-description"];
    }
    
    return tooltip;
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype._handlingMultiple = function(tooltip, paramPrefix, params)
{
    var count = parseInt(params[paramPrefix + "-content-size"]);
    if (count > 0)
    {
        if (tooltip != "")
        {
            tooltip += "<br/><br/>";
        }
        
        tooltip += this.getConfiguration()[paramPrefix + "-start-description"];
        for (var i=0; i < count; i++)
        {
            if (i != 0) 
            {
                tooltip += ", ";
            }
            tooltip += params[paramPrefix + "-content-" + i];
        }
        tooltip += this.getConfiguration()[paramPrefix + "-end-description"];
    }
    
    return tooltip;
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype._handlingMultipleSeveralScheduled = function(tooltip, params)
{
    if (tooltip != "")
    {
        tooltip += "<br/><br/>";
    }
    
    tooltip += this.getConfiguration()["scheduled-archiving-several-start-description"];
    tooltip += params["scheduled-archiving-several-nearest-date"];
    tooltip += this.getConfiguration()["scheduled-archiving-several-end-description"];
        
    return tooltip;
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype._getIcon = function(isScheduled)
{
    var index;
    if (!isScheduled)
    {
        index = this.getConfiguration()["non-toggled-item-index"];
    }
    else
    {
        index = this.getConfiguration()["toggled-item-index"];
    }
    
    var iconSmall = context.contextPath + this.getConfiguration()["menu-" + index + "-iconSmall"];
    var iconLarge = context.contextPath + this.getConfiguration()["menu-" + index + "-iconMedium"];
        
    this.getElements().each(
            function (element) 
            {
                element.iconSmall = iconSmall;
                element.iconLarge = iconLarge;
            }
    );
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype.configurationError = function(response)
{
    this.getElements().each(
        function(element) 
        { 
            element.setDisabled(true);
            element.toggle(false);
            element.stopRefreshing();
        }
    );
    this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.cms.buttons.archive.ArchiveMenu.prototype.configurationChanged = function(params)
{
    this.getElements().each(function(element) { element.stopRefreshing(); });
    
    var count = parseInt(params["allright-content-size"]);
    var scheduledCount = parseInt(params["scheduled-archiving-content-size"]);
    var hasOneContent = count > 0;
    var hasOneScheduled = scheduledCount > 0;
    var hasSeveralScheduled = scheduledCount > 1;
    
    this._allRightContentsId = [];
    if (hasOneContent)
    {
        for (var i=0; i < count; i++)
        {
            this._allRightContentsId.push(params["allright-content-" + i + "-id"]);
        }
    }
    
    // tooltip
    var tooltip = this._allRightTooltip("", "allright", params, hasOneScheduled);
    
    if (!hasOneContent)
    {
        tooltip += this.getConfiguration()['error-description'];
    }
    
    tooltip = this._handlingMultiple(tooltip, "locked", params);
    tooltip = this._handlingMultiple(tooltip, "noright", params);
    tooltip = this._handlingMultiple(tooltip, "nomodifiable", params);
    tooltip = this._handlingMultiple(tooltip, "workflowaction", params);
    tooltip = this._handlingMultiple(tooltip, "workflowstep", params);
    
    if (hasSeveralScheduled)
    {
        tooltip = this._handlingMultipleSeveralScheduled(tooltip, params);
    }
    else
    {
        tooltip = this._handlingMultiple(tooltip, "scheduled-archiving", params);
    }
    
    // icon
    this._getIcon(hasOneScheduled);
    this.getElements().each(function(element) { element.updateIcon(); });
    this.updateTooltip(tooltip, this.getElements().get(0).iconLarge, null);
    
    // toggle + disable
    var toggleState = hasOneScheduled;
    var disableState = count <= 0;
    this.getElements().each(
        function(element) 
        { 
            element.setDisabled(disableState);
            element.toggle(toggleState);
        }
    );
}
